/* -*- Mode: java; tab-width: 8; c-basic-offset: 4 -*-
 * Copyright  1997, 1998 Netscape Communications Corporation, All Rights Reserved.
 */

package com.netscape.javascript.CORBA;
import  com.netscape.javascript.*;
import  java.util.Hashtable;

import  org.omg.CORBA.CompletionStatus;

//<VENDOR_SPECIFIC VENDOR=Visigenic AUTHOR=ruslan>
import  com.visigenic.vbroker.URLNaming.*;
//</VENDOR_SPECIFIC>

/**
 * This class handles CORBA service initialization.
 *
 * @author Mike Shaver
 */

public class Init {

    public static void init (Scriptable scope)
        throws com.netscape.javascript.IllegalThreadAssociationException
    {
	orb = org.omg.CORBA.ORB.init();
	boa = orb.BOA_init ();   // needed for DSI

        pending_exceptions = new Hashtable ();
        thrown_exceptions  = new Hashtable ();

	// OPT: do this _very_ lazily
	rep = org.omg.CORBA.RepositoryHelper.bind (orb);

        //<VENDOR_SPECIFIC VENDOR=Visigenic AUTHOR=ruslan>
        try {
            url_resolver = ResolverHelper.narrow (
                orb.resolve_initial_references ("URLNamingResolver")
            );
        }
	catch (org.omg.CORBA.UserException se)
	{
            ResolverImpl ri = new ResolverImpl ();
	    ri.init ((com.visigenic.vbroker.orb.ORB)orb, null);
	    url_resolver = ri;
	};
        //</VENDOR_SPECIFIC>

	Scriptable CORBAroot = new NativeObject();
	CORBAroot.setParentScope(scope);
	CORBAroot.setPrototype(ScriptableObject.getObjectPrototype(scope));
	scope.put ("CORBA", scope, CORBAroot);

	// XXX CORBA.Converter?
	Context cx;
	try {
            cx = Context.getCurrentContext();
	}
	catch (IllegalThreadAssociationException e) {
	    // XXX
	}
	CORBAroot.addFunctionProperties(CORBA_methods, Init.class, ScriptableObject.DONTENUM);

	try {
	    ScriptableObject.defineClass(CORBAroot, Module.class);
	    ScriptableObject.defineClass(CORBAroot, CORBAObject.class);
	} 
	catch (Exception e) 
	{
	    cx.getErrorReporter().error("failed to initialize CORBA services:" +
					e.getMessage(), "*CORBA initialization*",
					1);
	}
    }

    public static Object resolveURL (String url)
    {
        org.omg.CORBA.Object obj;

        try {
            obj = url_resolver.locate (url);
        }
        catch (Exception e)
        {
            // ruslan: until exceptions are implemented
      	    throw new JavaScriptException ("CORBA Object initialization failure: unable to resolve URL - " + url + " (" + e + ")");
        }

        return new CORBAObject (obj);
    }

    public static Object resolve (String ior)
    {
    	try {
            org.omg.CORBA.Object obj = orb.string_to_object (ior);
            return new CORBAObject (obj);
        }
        catch (org.omg.CORBA.INV_OBJREF e)
        {
            // ruslan: until exceptions are implemented
	    throw new JavaScriptException ("CORBA Object initialization failure: invalid IOR presented");
        }
    }

    public synchronized static boolean pendingException ()
    {
        return pending_exceptions.get (Thread.currentThread ()) != null;
    }

    public synchronized static Object getException ()
    {
        Thread t = Thread.currentThread ();
        Object p = pending_exceptions.get (t);

        if (p != null)
                pending_exceptions.remove (t);
        return p;
    }

    public synchronized static Object getThrownException ()
    {
        Thread t = Thread.currentThread ();
        Object p = thrown_exceptions.get (t);

        if (p != null)
                thrown_exceptions.remove (t);
        return p;
    }

    public synchronized static void throwException (Object value)
    {
        Thread t = Thread.currentThread ();

        if (value instanceof Struct)
            thrown_exceptions.put (t, value);
        else    {
            thrown_exceptions.remove (t);
            raise (new SystemException ("BAD_PARAM", "valid exception type expected", CompletionStatus.COMPLETED_NO));
        }
    }

    public synchronized static void raise (Object p)
    {
        pending_exceptions.put (Thread.currentThread (), p);
    }

    static org.omg.CORBA.ORB orb;
    static org.omg.CORBA.BOA boa;
    static org.omg.CORBA.Repository rep;

    static Hashtable pending_exceptions;
    static Hashtable thrown_exceptions;    // used in DSI

    static String [] CORBA_methods = { "resolve", "resolveURL", "pendingException", "getException", "throwException" };

    //<VENDOR_SPECIFIC VENDOR=Visigenic AUTHOR=ruslan>
    static Resolver url_resolver;
    //</VENDOR_SPECIFIC>

}
